#! /usr/bin/env python
# test_json.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

import unittest
from tralchemy.json import resource, query, insert


class TestJson(unittest.TestCase):

    def test_simple(self):
        q = query({
            "name": "file",
            "type": "nfo:FileDataObject"
            })
        q.get_sparql()

    def test_relation_shorthand(self):
        q = query({
            "name": "photo",
            "type": "nfo:Image",
            "nie:isStoredAs": "?storedas"
            })
        q.get_sparql()

    def test_relation(self):
        q = query({
            "name": "photo",
            "type": "nfo:Image",
            "nie:isStoredAs": {
               "name": "storedas",
               },
            })
        q.get_sparql()

    def test_resource_relation(self):
        q = query({
            "name": "badger",
            "type": "nfo:isStoredAs",
            "rdf:type": resource("http://www.test.com/"),
            })
        q.get_sparql()

    def test_offset(self):
        q = query({
            "name": "photo",
            "type": "nfo:Image",
            "offset": 5
            })
        q.get_sparql()

    def test_limit(self):
        q = query({
            "name": "photo",
            "type": "nfo:Image",
            "limit": 1,
            })
        q.get_sparql()

    def test_orderby(self):
        q = query({
            "name": "uri",
            "type": "nco:Contact",
            "nco:fullname": "?fullname",
            "order": ["fullname"],
            })
        q.get_sparql()

    def test_orderby_2(self):
        q = query({
            "name": "uri",
            "type": "nco:Contact",
            "nco:fullname": "?fullname",
            "order": [{"name": "fullname", "descending": False}],
            })
        q.get_sparql()

    def test_fetch_pred_obj_for_sub(self):
        q = query({
            "name": resource("yourface"),
             "?a": "?b"
            })
        q.get_sparql()

    def test_update(self):
        u = insert({
            "type": "nco:Contact",
            "nco:fullname": "John Carr",
            })
        u.get_sparql()

    def test_update_existing(self):
        u = insert({
            "resource": "urn:uuid:123456",
            "nco:fullname": "John Carr",
            })
        u.get_sparql()
