#! /usr/bin/env python
# test_property.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

import unittest
from tralchemy.property import Property
from tralchemy.ns.rdfs import Class
from tralchemy.ns.xsd import string
from tralchemy.ns.nid3 import ID3Audio


class TestProperty(unittest.TestCase):

    def test_get(self):
        """ Should be able to get properties """
        obj = Class("nid3:ID3Audio")
        a = obj.modified

    def test_set(self):
        """ Should be able to set properties """
        obj = Class("nid3:ID3Audio")
        obj.notify = True

    def test_help(self):
        """ Poke help """
        self.failIfEqual(ID3Audio.leadArtist.__doc__, None)

    def test_types_string(self):
        """ Test types of properties """
        # If we ask wrapper for a string, we should get a string type
        self.failUnlessEqual(string, str)

        # cls1 is a class, so accessing properties on it will return self rather then actually doing anything in __get__
        self.failUnlessEqual(type(ID3Audio.leadArtist), Property)

