#! /usr/bin/env python
# test_property_list.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

import unittest
from tralchemy.ns.rdfs import Class
from tralchemy.ns.nco import PersonContact, PhoneNumber


class TestPropertyList(unittest.TestCase):

    def test_len(self):
        """ Should be able to get the length of a list property """
        foo = Class("nco:PersonContact")
        self.failUnlessEqual(len(foo.subClassOf), 1)
        foo = Class("ncal:Event")
        self.failUnlessEqual(len(foo.subClassOf), 2)

    def test_append(self):
        """ Should be able to append to a lst property """
        p = PersonContact.create()
        for i in range(5):
            pn = PhoneNumber.create(phoneNumber=str(i))
            p.hasPhoneNumber.append(pn)
        p.commit()
        self.failUnlessEqual(len(p.hasPhoneNumber), 5)
        return p.uri

    def test_list(self):
        """ Should be able to retrieve a list from a property """
        uri = self.test_append()
        p = PersonContact(uri)
        i = 0
        k = 0
        for j in p.hasPhoneNumber:
            i += 1
            k += int(j.phoneNumber)
        self.failUnlessEqual(i, 5)
        self.failUnlessEqual(k, (4+3+2+1+0))

    def test_dont_assign_to_list(self):
        """ Don't support directly assigning to a list property """
        #FIXME: In the future, you might be able to accept a list/set/tuple, and this test will
        # need to be relaxed
        cls = Class("rdfs:Class")
        self.failUnlessRaises(ValueError, setattr, cls, "subClassOf", "yourface")


