#! /usr/bin/env python
# test_records.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

import unittest, uuid
from tralchemy.ns.rdfs import Class
from tralchemy.ns.nco import Contact

class TestRecords(unittest.TestCase):

    def test_inject_contacts(self):
        """ Should be able to commit objects """
        c = Contact("http://localhost/contact/%s" % str(uuid.uuid4()))
        #today = datetime.datetime.today()
        #date = today.isoformat() + "+00:00"
        #feedmsg.contentlastmodified = date
        #feedmsg.communicationchannel = "http://planet.gnome.org/atom.xml"
        #feedmsg.title = "Your face"
        c.fullname = "Test"
        c.commit()

    def test_delete(self):
        """ Should be able to delete objects """
        self.test_inject_contacts()
        a = 0
        for msg in Contact.get():
            a += 1
            msg.delete()
        self.failUnless(a > 0)
        b = 0
        for msg in Contact.get():
            b += 1
        self.failUnlessEqual(b, 0)

    def test_iteritems(self):
        self.test_inject_contacts()

        for row in Contact.get(fullname="test"):
            _name = False
            _type = False
            for key, value in row.iteritems():
                if key =="nco:fullname":
                    assert isinstance(value, basestring)
                    _name = True
                elif key == "rdf:type":
                    assert isinstance(value, list)
                    _type = True

            self.assertEquals(_name, True)
            self.assertEquals(_type, True)

    def test_get_with_criteria(self):
        """ Should be able to fetch a subset of the store objects """
        self.failUnless(len(list(Class.get())) > len(list(Class.get(notify="true"))))


