# Backends
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>


class Result(object):

    def __init__(self):
        self.contents = {}

    def add(self, key, value):
        self.contents[key] = value

    def get_value(self, key):
        if key in self.contents:
            return self.contents[key]
        return None

    def get_keys(self):
        return self.contents.keys()

    def iteritems(self):
        for k, v in self.contents.iteritems():
            yield k, v

    def __getitem__(self, idx):
        return self.get_value(idx)

    def __contains__(self, val):
        return val in self.contents


class Results(object):

    def __init__(self, backend, query, replace=None, async=True):
        self.backend = backend
        self.query = query
        self.async = async

        self.keys = {}
        self.results = None
        self.inserted = None
        self.removed = None

        self.keysize = 0
        for keyname in query.get_keys():
            self.keys[keyname] = self.keysize
            self.keysize += 1

        self.sparql = query.get_sparql()
        if replace:
            self.sparql = self.sparql % replace

        self._query_execute()

    def _query_execute(self):
        raise NotImplementedError

    def _query_reply(self, results):
        self.results = results

        if self.inserted:
            for i in range(len(results)):
                self.inserted(i)

    def n_children(self):
        if self.results is None:
            return 0
        return len(self.results)

    def get_value(self, index):
        if self.keysize == 1 and len(self.query.subqueries) == 0:
            return unicode(self.results[index][0])

        r = Result()

        for key, idx in self.keys.iteritems():
            r.add(key, unicode(self.results[index][idx]))

        for subquery in self.query.subqueries:
            idx = self.keys[subquery.where]
            r.add(subquery.at, Results(subquery.query, results[index][idx]))

        return r

    def __iter__(self):
        for i in range(self.n_children()):
            yield self.get_value(i)


class Backend(object):

    def query(self, query):
        raise NotImplementedError

    def update(self, query):
        raise NotImplementedError

    def get_results(self, query, async=True):
        raise NotImplementedError


def set_backend(backend):
    import tralchemy
    import sys
    tralchemy.endpoint = backend
    sys.modules["tralchemy.endpoint"] = backend

