# tracker.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

from __future__ import absolute_import
import dbus

from . import Results as R
from . import Backend as B

class Results(R):

    def _query_execute(self):
        self.bus = dbus.SessionBus()
        self.proxy = self.bus.get_object('org.freedesktop.Tracker1', '/org/freedesktop/Tracker1/Resources')
        self.tracker = dbus.Interface(self.proxy, 'org.freedesktop.Tracker1.Resources')

        if self.async:
            self.tracker.SparqlQuery(self.sparql, reply_handler=self._query_reply, error_handler=self._query_error)
        else:
            results = self.tracker.SparqlQuery(self.sparql)
            self._query_reply(results)

    def _query_error(self, err):
        pass


class Endpoint(B):

    def __init__(self):
        bus = dbus.SessionBus()
        tracker_obj = bus.get_object("org.freedesktop.Tracker1", "/org/freedesktop/Tracker1/Resources")
        self.tracker = dbus.Interface(tracker_obj, "org.freedesktop.Tracker1.Resources")

    def query(self, sparql):
        try:
            sparql = sparql.replace('\n', '\\n')
            return self.tracker.SparqlQuery(sparql)
        except Exception as detail:
            raise ValueError("Invalid SPARQL: %s. Detail: %s" % (sparql, detail))

    def update(self, sparql):
        try:
            sparql = sparql.replace('\n', '\\n')
            return self.tracker.SparqlUpdate(sparql)
        except Exception as detail:
            raise ValueError("Invalid Update SPARQL: %s. Detail: %s" % (sparql, detail))

    def get_results(self, query, async=True):
        return Results(self, query, async=async)


def install():
    b = Endpoint()
    from . import set_backend
    set_backend(b)

