
from __future__ import absolute_import

# Import gobject to init the type system
# and turn threads on
import gobject
gobject.threads_init()

import os
from ctypes import *

from . import Results as R
from . import Backend as B

libc = CDLL("libc.so.6")
libglib = CDLL("libglib-2.0.so")
libtracker_data = None
libtracker_common = None

# Turn hacks to full :)
search_path = os.path.abspath(__file__).split(os.path.sep)
while search_path:
    joined = os.path.sep.join(search_path)
    joined = os.path.join(joined, "lib", "tracker-0.7")
    if os.path.exists(joined):
        libtracker_data = CDLL(os.path.join(joined, "libtracker-data.so"))
        libtracker_common = CDLL(os.path.join(joined, "libtracker-common.so"))
        break
    search_path.pop()
assert libtracker_data != None


TRACKER_DB_MANAGER_FORCE_REINDEX    = 1 << 1
TRACKER_DB_MANAGER_REMOVE_CACHE     = 1 << 2
TRACKER_DB_MANAGER_LOW_MEMORY_MODE  = 1 << 3
TRACKER_DB_MANAGER_REMOVE_ALL       = 1 << 4
TRACKER_DB_MANAGER_READONLY         = 1 << 5


def instancemethod(method):
    def _(self, *args, **kwargs):
        return method(self, *args, **kwargs)
    return _


class Error(c_void_p):
    pass


class ResultSet(c_void_p):

    n_rows = instancemethod(libtracker_data.tracker_db_result_set_get_n_rows)
    n_columns = instancemethod(libtracker_data.tracker_db_result_set_get_n_columns)
    get = instancemethod(libtracker_data.tracker_db_result_set_get)
    next = instancemethod(libtracker_data.tracker_db_result_set_iter_next)

    def get_string(self, column):
        result = c_void_p()
        self.get(column, byref(result), -1)
        if not result:
           return None
        return string_at(result)

    def get_row(self):
        #FIXME: This is a little inefficient but will do for now
        result = []
        for i in range(self.n_columns()):
            result.append(self.get_string(i))
        return result

    def __iter__(self):
        valid = True
        while valid:
            yield self.get_row()
            valid = self.next()


class Results(R):

    def _query_execute(self):
        rs = self.backend.query(self.sparql)
        self._query_reply([r for r in rs])


libtracker_data.tracker_db_result_set_get_n_columns.argtypes = [ResultSet]
libtracker_data.tracker_db_result_set_get_n_columns.restype = c_int

libtracker_data.tracker_db_result_set_get_n_rows.argtypes = [ResultSet]
libtracker_data.tracker_db_result_set_get_n_rows.restype = c_int

#FIXME: This declaration limits us to one at a time...
libtracker_data.tracker_db_result_set_get.argtypes = [ResultSet, c_int, POINTER(c_void_p), c_int]
libtracker_data.tracker_db_result_set_get.restype = None

libtracker_data.tracker_db_result_set_iter_next.argtypes = [ResultSet]
libtracker_data.tracker_db_result_set_iter_next.restype = c_bool

libtracker_data.tracker_data_query_sparql.argtypes = [c_char_p, POINTER(Error)]
libtracker_data.tracker_data_query_sparql.restype = ResultSet

libtracker_data.tracker_data_manager_init.argtypes = [c_int, c_char_p, c_void_p, c_void_p]
libtracker_data.tracker_data_manager_init.restype = None

libtracker_data.tracker_data_manager_shutdown.argtypes = []
libtracker_data.tracker_data_manager_shutdown.restype = None

libtracker_common.tracker_log_init.argtypes = [c_int, POINTER(c_char_p)]
libtracker_common.tracker_log_init.restype = c_bool

class Backend(object):

    def __init__(self, location=None):
        if location:
            os.environ['XDG_DATA_HOME'] = location
            os.environ['XDG_CACHE_HOME'] = location
        libtracker_common.tracker_log_init(0, None)
        libtracker_data.tracker_data_manager_init(0, None, None, None)

    def __del__(self):
        libtracker_data.tracker_data_manager_shutdown()

    def query(self, query):
        err = Error()
        rs = libtracker_data.tracker_data_query_sparql(query, byref(err))
        if err:
            raise ValueError("Invalid query")
        if not rs:
            return []
        return [x for x in rs]

    def update(self, query):
        err = Error()
        libtracker_data.tracker_data_update_sparql(query, byref(err))
        if err:
            raise ValueError("Invalid query")

    def get_results(self, query, async=True):
        return Results(self, query, async)


def install(location=None):
    b = Backend(location=location)
    from . import set_backend
    set_backend(b)
