# json.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

import uuid
from ast import *

class V(object):

    def __init__(self, name):
        self.v = name
        self.test = None
        self.target = None

    def equiv(self, target):
        self.test = "is"
        self.target = target

    def equals(self, target):
        self.test = "="
        self.target = target

    def get_filter(self, query):
        a = query.get_variable(self.v)
        b = self.test
        c = None

        if isinstance(self.target, basestring):
            if self.target.startswith("?"):
                c = query.get_variable(self.target[1:])
            else:
                if this.test == "is":
                    b = "="
                    c = ConstantResource(self.target)
                else:
                    c = Constant(self.target)

        return Filter(a, b, c)

def resource(object):
    return ConstantResource(object)

def query(obj):
    q = Query()

    state = {"unnamed": 0}

    def parse(obj, container, state):
        v = None
        va = None
        select = True

        if not "name" in obj:
            select = False
            obj["name"] = "a" + unicode(state["unnamed"])
            state["unnamed"] += 1

        if "select" in obj:
            select = obj["select"]

        if not isinstance(obj["name"], ConstantResource):
            if q.has_variable(obj["name"]):
                va = q.get_variable(obj["name"])
            else:
                va = Variable(obj["name"])
                if select:
                    q.add(Select(va))
                q.add_variable(va)
        else:
            va = obj["name"]

        if "type" in obj:
            v = RelationSet(va, PrefixedName(obj["type"]))
        else:
            v = RelationSet(va)

        props = 0
        for key, value in obj.iteritems():
            if key in ("name", "type", "select", "optional", "order", "badgers", "filters"):
                continue

            if key.startswith("?"):
                k = Variable(key[1:])
                q.add(Select(k))
                q.add_variable(k)
            else:
                k = PrefixedName(key)

            props += 1

            if isinstance(value, dict):
                if "optional" in value and value["optional"] == True:
                    o = Optional()
                    o.add(Relation(va, k, parse(value, o, state)))
                    q.add(o)
                else:
                    v.add(k, parse(value, container, state))
            elif isinstance(value, basestring):
                if value.startswith("?"):
                    v.add(k, parse({"name": value[1:]}, container, state))
                else:
                    v.add(k, Constant(value))
            elif isinstance(value, Constant):
                v.add(k, value)
            else:
                v.add(k, Constant(value))

        if props > 0 or "type" in obj:
            q.add(v)

        return va

    parse(obj, q, state)

    if "badgers" in obj:
        for badger in obj["badgers"]:
            if "optional" in badger and badger["optional"] == True:
                o = Optional
                parse(badger, o, state)
                q.add(o)
            else:
                parse(badger, q, state)

    if "order" in obj:
        for order in obj["order"]:
            if isinstance(order, basestring):
                q.add_order(q.get_variable(order), False)
            elif isinstance(order, dict):
                q.add_order(q.get_variable(order["name"]), order["descending"])

    if "offset" in obj:
        q.offset = obj["offset"]

    if "limit" in obj:
        q.limit = obj["limit"]

    if "filters" in obj:
        for filter in obj["filters"]:
            q.add(filter.get_filter(q))

    return q


def insert(obj):
    u = Insert()

    def parse(obj):
        if not ("resource" in obj or "type" in obj):
            raise ValueError("No subject to update and no type to create a new one")

        res = obj["resource"] if "resource" in obj else "urn:uuid:" + unicode(uuid.uuid4())
        typ = PrefixedName(obj["type"]) if "type" in obj else None

        rs = RelationSet(ConstantResource(res), typ)
        u.add(rs)

        for k, v in obj.iteritems():
            if k in ("resource", "type"):
                continue

            if v is None:
                raise ValueError("'%s' cannot be None" % k)
            elif isinstance(v, dict):
                rs.add(k, ConstantResource(parse(v)))
            elif isinstance(v, list):
                for l in v:
                    if isinstance(l, Constant):
                        rs.add(k, l)
                    else:
                        rs.add(k, Constant(l))
            elif isinstance(v, Constant):
                rs.add(k, v)
            else:
                rs.add(k, Constant(v))

        return res

    parse(obj)

    return u

def delete(obj):
    d = Delete()

    def parse(dict):
        if not "resource" in dict:
            raise ValueError("You must specify a specific resource to delete triples for")

        rs = RelationSet(ConstantResource(dict["resource"]))
        d.add_where(rs)

        if "delete" in dict and dict["delete"]:
            d.add_delete(rs)

        for k, v in dict.iteritems():
            if k in ("resource", "type", "delete"):
                continue
            if v is None:
                raise ValueError("'%s' cannot be None" % k)
            elif isinstance(v, basestring):
                if v.startswith("?"):
                    rs.add(k, Variable(v[1:]))
                else:
                    rs.add(k, Constant(v))
            else:
                rs.add(k, Constant(v))

    parse(obj)

    return d

def sparql(sparql):
    return SparqlUpdate(sparql)

def transaction(*updates):
    return Transaction(*updates)
