# property.py
#
# Copyright (C) 2009, Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#
# Author:
#       John Carr <john.carr@unrouted.co.uk>

from .utils import get_classname
from . import endpoint
from .json import resource

class PropertyList(object):

    def __init__(self, uri, range, prop, instance):
        self.uri = uri
        self._range_ = range
        self.property = prop
        self.instance = instance
        self.vals = []

        instance_uri = instance.uri
        if instance_uri.startswith("http://") or instance_uri.startswith("urn:"):
            instance_uri = "<%s>" % instance_uri

        # Get all items of this type
        for result in endpoint.query("SELECT ?o WHERE { %s %s ?o }" % (instance_uri, uri)):
            result = result[0]
            self.vals.append(self.property.factory.get_class(range)(result))

    def append(self, value):
        eval = value
        self.instance.triples.setdefault(self.uri, []).append(eval)
        self.vals.append(value)

    def __delitem__(self, idx):
        raise NotImplementedError

    def __setitem__(self, idx, value):
        raise NotImplementedError

    def __getitem__(self, idx):
        return self.vals[idx]

    def __len__(self):
        return len(self.vals)

    def __iter__(self):
        return iter(self.vals)

    def __repr__(self):
        return repr(self.vals)

    def __str__(self):
        return unicode(self.vals)


class Property(property):

    def __init__(self, uri, factory):
        self.uri = uri
        self.factory = factory

        sparql = "SELECT ?max ?range ?comment WHERE { " \
                     "%s a rdf:Property . " \
                     "OPTIONAL { %s nrl:maxCardinality ?max } . " \
                     "OPTIONAL { %s rdfs:range ?range } . " \
                     "OPTIONAL { %s rdfs:comment ?comment } " \
                 "}"

        results = endpoint.query(sparql % (uri, uri, uri, uri))
        for result in results:
            self._maxcard_ = int(result[0]) if result[0] else None
            self._range_ = result[1]
            self.__doc__ = "%s\n\n@type: %s" % (result[2], get_classname(self._range_))

    def is_list(self):
        return not self._maxcard_ or self._maxcard_ > 1

    def __get__(self, instance, instance_type):
        if instance is None:
            return self

        # Just so pychecker doesnt moan
        assert instance_type

        if self.is_list():
            if not self.uri in instance.cache:
                instance.cache[self.uri] = PropertyList(self.uri, self._range_, self, instance)
            return instance.cache[self.uri]

        uri = instance.uri
        if uri.startswith("http://"):
            uri = "<%s>" % uri

        results = endpoint.query("SELECT ?v WHERE { %s %s ?v }" % (uri, self.uri))
        for result in results:
            #FIXME: What to do about lists of stuff. What to do about stuff that isnt a string.
            result = result[0]
            if self.uri == "rdfs:range":
                return unicode(result)
            else:
                return self.factory.get_class(self._range_)(result)

    def __set__(self, instance, value):
        if instance is None:
            return

        if self.is_list():
            raise ValueError("Cannot assign directly to a list property")

        instance.triples[self.uri] = value

    def __delete__(self, instance):
        pass

